"""
bioBakery Workflows: tasks.files module
A collection of file names used by tasks

Copyright (c) 2017 Harvard School of Public Health

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import os
import copy
import sys

from anadama2 import reporters

from .utilities import name_files

class FileInfo(object):
    def __init__(self, name=None, subfolder=None, tag=None, extension=None, description=None):
        # set a list of non-path keywords
        self.non_path_keywords=["description"]
        
        # concat multiple strings if present in description
        if description and isinstance(description,tuple):
            description="\n".join(description)
        
        keywords={"names":name, "subfolder":subfolder, "tag":tag, "extension":extension, "description":description}
        self.keywords = {key:value for key, value in keywords.items() if value}
        
    def get_path_keywords(self):
        info = copy.copy(self.keywords)
        
        # remove the non-path keywords
        for key in self.non_path_keywords:
            try:
                del info[key]
            except KeyError:
                pass
        
        return info
    
    def __getitem__(self, key):
        """ Return the file info """
        try:
            value=self.keywords[key]
        except KeyError:
            value=""
        return value

class Workflow(object):
    file_info = {}
    
    file_info["log"]=FileInfo(reporters.LOG_FILE_NAME,
        description="The AnADAMA2 workflow log.")
    
    @classmethod
    def path(cls, name, main_folder="", none_if_not_found=None, error_if_not_found=None, **keywords):
        merged_keywords = copy.copy(keywords)
        merged_keywords.update(cls.file_info[name].get_path_keywords())
        file_path=name_files(folder=main_folder, **merged_keywords)

        # if set, error if the file does not exist
        if error_if_not_found and not os.path.isfile(file_path):
            message="\nERROR: Unable to find file: "+file_path
            desc=cls.description(name)
            if desc:
                message+="\n\nFile description:\n"+desc
            sys.exit(message)
        
        # if set, check if the file exists, if not return None
        if none_if_not_found and not os.path.isfile(file_path):
                file_path = None
            
        return file_path
    
    @classmethod
    def description(cls,name):
        try:
            desc=cls.file_info[name].keywords["description"]
        except (KeyError, AttributeError):
            desc=""
            
        return desc
    
    @classmethod
    def list_file_path_description(cls,folder,input_files):
        """ List the file names and descriptions in a format to be used in an argument help description """
        
        desc=""
        for required in input_files:
            desc+="\n\n".join(["* "+cls.path(name,folder)+ " ( " + required + " )\n-- "+cls.description(name) for name in input_files[required]])+"\n"
            
        return desc

class ShotGun(Workflow):
    """ A collection of information of folders/files created by the shotgun tasks """
    
    file_info=copy.copy(Workflow.file_info)
    
    # set the folder names for wmgx_wmtx data workflows
    wmgx_folder_name="whole_metagenome_shotgun"
    wmtx_folder_name="whole_metatranscriptome_shotgun"
    
    # set the kneaddata file name
    file_info["kneaddata_read_counts"]=FileInfo("kneaddata_read_count_table.tsv",subfolder=os.path.join("kneaddata","merged"),
        description=("A tab-delimited file with samples as rows and read counts as columns.",
            "This file is generated by compiling information from the KneadData log files. ",
            "The file contains read counts after each step of the Kneaddata workflow. ",
            "Depending on the input to Kneaddata this file will contain counts for ",
            "single or paired/orphan reads. The read counts from the filtering steps ",
            "are for filtering applied in a serial manner with the reads that do not ",
            "map to the first reference used as input in filtering with the next ",
            "reference database continuing until all databases have been run."))

    # set the taxonomy file names
    file_info["taxonomic_profile"]=FileInfo("metaphlan2_taxonomic_profiles.tsv",subfolder=os.path.join("metaphlan2","merged"),
        description=("A tab-delimited file with samples as columns and relative abundance as rows.",
            "This file contains the merged taxonomic profiles computed by MetaPhlAn2 for all samples."))
    file_info["species_counts"]=FileInfo("metaphlan2_species_counts_table.tsv",subfolder=os.path.join("metaphlan2","merged"),
        description=("A tab-delimited file with samples as rows and counts as columns.",
            "This file contains the counts of total species for each sample using the ",
            "species identified by MetaPhlAn2."))
    
    # set the merged feature file names
    file_info["genefamilies"]=FileInfo("genefamilies.tsv",subfolder=os.path.join("humann2","merged"),
        description=("A tab-delimited file with samples as columns and gene families ",
            "as rows. This file is a merged set of gene families for all samples ",
            "computed by HUMAnN2. This file contains stratified counts as RPKs."))
    file_info["ecs"]=FileInfo("ecs.tsv", subfolder=os.path.join("humann2","merged"),
        description=("A tab-delimited file with samples as columns and ecs as rows. ",
            "This file is a merged set of ecs for all samples generated from the gene ",
            "families computed by HUMAnN2. This file contains stratified counts as RPKs."))
    file_info["pathabundance"]=FileInfo("pathabundance.tsv", subfolder=os.path.join("humann2","merged"),
        description=("A tab-delimited file with samples as columns and pathways ",
            "as rows. This file is a merged set of pathway abundances for all ",
            "samples computed by HUMAnN2. This file contains stratified counts ",
            "of non-normalized abundances."))
    
    # set the normed feature file names
    file_info["genefamilies_relab"]=FileInfo("genefamilies_relab.tsv", subfolder=os.path.join("humann2","merged"))
    file_info["ecs_relab"]=FileInfo("ecs_relab.tsv", subfolder=os.path.join("humann2","merged"))
    file_info["pathabundance_relab"]=FileInfo("pathabundance_relab.tsv", subfolder=os.path.join("humann2","merged"),
        description=("A tab-delimited file with samples as columns and pathways ",
                "as rows. This file is a merged set of pathway abundances for all ",
                "samples computed by HUMAnN2. This file contains stratified counts ",
                "of relative abundances."))
    
    # set the feature count file names
    file_info["genefamilies_relab_counts"]=FileInfo("humann2_genefamilies_relab_counts.tsv", subfolder=os.path.join("humann2","counts"))
    file_info["ecs_relab_counts"]=FileInfo("humann2_ecs_relab_counts.tsv", subfolder=os.path.join("humann2","counts"))
    file_info["pathabundance_relab_counts"]=FileInfo("humann2_pathabundance_relab_counts.tsv", subfolder=os.path.join("humann2","counts"))
    
    # set the all feature counts file names
    file_info["feature_counts"]=FileInfo("humann2_feature_counts.tsv", subfolder=os.path.join("humann2","counts"),
        description=("A tab-delimited file with samples as rows and features ",
            "as columns. This file includes the total feature counts (non-stratified)",
            "for the features computed by HUMAnN2 (genes, ecs, and pathways)."))
    file_info["humann2_read_counts"]=FileInfo("humann2_read_and_species_count_table.tsv", subfolder=os.path.join("humann2","counts"),
        description=("A tab-delimited file with samples as rows and counts as columns.",
            "This file was created using the HUMAnN2 logs. It includes the total number ",
            "of species used to generate the custom database, the total number of initial",
            "reads, and the total reads aligning for both search steps."))
    
    # set the names for the rna/dna normed files
    file_info["genefamilies_norm_ratio"]=FileInfo("rna_dna_relative_expression_unstratified.tsv",subfolder=os.path.join("humann2","rna_dna_norm","genes"),
        description=("A tab-delimited file with samples as columns and genes as rows. ",
            "This file includes the normalized RNA abundances as a ratio to DNA abundance. ",
            "This file does not include stratified features."))
    file_info["ecs_norm_ratio"]=FileInfo("rna_dna_relative_expression_unstratified.tsv",subfolder=os.path.join("humann2","rna_dna_norm","ecs"),
        description=("A tab-delimited file with samples as columns and ecs as rows. ",
            "This file includes the normalized RNA abundances as a ratio to DNA abundance. ",
            "This file does not include stratified features."))
    file_info["paths_norm_ratio"]=FileInfo("rna_dna_relative_expression_unstratified.tsv",subfolder=os.path.join("humann2","rna_dna_norm","paths"),
        description=("A tab-delimited file with samples as columns and pathways as rows. ",
            "This file includes the normalized RNA abundances as a ratio to DNA abundance. ",
            "This file does not include stratified features."))
    
class ShotGunVis(Workflow):
    """ A collection of information of folders/files created by the shotgun vis templates """
    
    file_info=copy.copy(Workflow.file_info)
    
    file_info["microbial_counts"]=FileInfo("microbial_counts_table.tsv",
        description="A tab-delimited file with samples as rows and ratios as "+\
            "columns. Includes the proportion of reads remaining after "+\
            "trimming and filtering in the quality control workflow.")
    file_info["rna_microbial_counts"]=FileInfo("rna_microbial_counts_table.tsv",
        description="A tab-delimited file with RNA samples as rows and ratios as "+\
            "columns. Includes the proportion of reads remaining after "+\
            "trimming and filtering in the quality control workflow.")
    file_info["qc_counts"]=FileInfo("qc_counts_table.tsv",
        description="A tab-delimited file with samples as rows and read counts "+\
            "as columns. Includes the read counts for trimming and filtering steps "+\
            "in the quality control workflow. The reads are single end.")
    file_info["qc_counts_paired"]=FileInfo("qc_counts_pairs_table.tsv",
        description="A tab-delimited file with samples as rows and read counts "+\
            "as columns. Includes the read counts for trimming and filtering steps "+\
            "in the quality control workflow. The reads are paired end and these "+\
            "counts are only for pairs.")
    file_info["qc_counts_orphan"]=FileInfo("qc_counts_orphans_table.tsv",
        description="A tab-delimited file with samples as rows and read counts "+\
            "as columns. Includes the read counts for trimming and filtering steps "+\
            "in the quality control workflow. The reads are paired end and these "+\
            "counts are only for orphans.")
    file_info["rna_qc_counts_paired"]=FileInfo("rna_qc_counts_pairs_table.tsv",
        description="A tab-delimited file with RNA samples as rows and read counts "+\
            "as columns. Includes the read counts for trimming and filtering steps "+\
            "in the quality control workflow. The reads are paired end and these "+\
            "counts are only for pairs.")
    file_info["rna_qc_counts_orphan"]=FileInfo("rna_qc_counts_orphans_table.tsv",
        description="A tab-delimited file with RNA samples as rows and read counts "+\
            "as columns. Includes the read counts for trimming and filtering steps "+\
            "in the quality control workflow. The reads are paired end and these "+\
            "counts are only for orphans.")
    file_info["taxa_counts"]=FileInfo("taxa_counts_table.tsv",
        description="A tab-delimited file with samples as rows and counts as "+\
            "columns. These are the total number of species/genera identified for each "+\
            "sample before and after filtering.")

class SixteenS(Workflow):
    """ A collection of information of folders/files created by the 16s tasks """
    
    file_info=copy.copy(Workflow.file_info)
    
    # set the names for the otu table and read count files
    file_info["otu_table_closed_reference"]=FileInfo("all_samples_taxonomy_closed_reference.tsv",
        description=("A tab-delimited file with samples/taxonomy as columns and taxonomy as rows. ",
            "First column is the OTU id and the last column is the taxonomy. The remaining",
            "columns are sample names. Values are counts."))
    file_info["otu_table_open_reference"]=FileInfo("all_samples_taxonomy_open_reference.tsv",
        description=("A tab-delimited file with samples/taxonomy as columns and taxonomy as rows. ",
            "First column is the OTU id and the last column is the taxonomy. The remaining",
            "columns are sample names. Values are counts. All OTUs without taxonomy are labeled Unclassified."))
    file_info["read_count_table"]=FileInfo("all_samples_read_counts.tsv",
        description=("A tab-delimited file with samples as rows and counts as columns. ",
            "The counts included are the original read count, total number of reads ",
            "mapping to an OTU with known taxonomy, and total reads mapping to an ",
            "unclassified OTU."))
    file_info["eestats2"]=FileInfo("all_samples_eestats2.txt",
        description=("A file with maxee as columns and read lengths as rows."))
    file_info["msa_nonchimera"]=FileInfo("all_samples_clustalo_aligned_nonchimera.fasta",
        description=("A multiple sequence alignment file generated from the nonchimera sequences ",
            "using Clustalo."))
    file_info["msa_closed_reference"]=FileInfo("all_samples_clustalo_aligned_closed_reference.fasta",
        description=("A multiple sequence alignment file generated from the closed reference sequences ",
            "using Clustalo."))
    # set the names for the otu tables and read count files
    file_info["error_ratesF"]=FileInfo("Error_rates_per_sample_FWD.png",
        description=("Plots of forward read error rates in dada2 workflow for each sample"))
    file_info["error_ratesR"]=FileInfo("Error_rates_per_sample_REV.png",
        description=("Plots of reverse read error rates in dada2 workflow for each sample"))
    file_info["readF_qc"]=FileInfo("FWD_read_plot.png",
        description=("Plots of quality of forward reads for each sample"))
    file_info["readR_qc"]=FileInfo("REV_read_plot.png",
        description=("Plots of quality of reverse reads for each sample"))
    file_info["counts_each_step"]=FileInfo("Read_counts_at_each_step.tsv",
        description=("A tab-delimited file with samples as rows and counts as columns. ",
            "The counts included in each step of workflow process"))
    file_info["filtN"]=FileInfo("filtN",
        description=("Folder with N filtered files"))

        




import unittest
import tempfile
import os
import sys

from biobakery_workflows import utilities

# write to a temp file
def write_temp(data):
    """ Write the data to a temp file """
    
    handle, file = tempfile.mkstemp(prefix="biobakery_workflows_test")
    os.write(handle,data)
    os.close(handle)
    
    return file

class TestUtiltiesFunctions(unittest.TestCase):
    """ Test the functions found in the biobakery workflows utilities module """
    
    def test_taxa_remove_unclassified(self):
        """ Test the taxa remove unclassified function """
        
        taxa=["k__k1;p__p1;c__","k__k1;p__p1;c__c1;o__o1;f__f1;g__g1;s__s1",
              "k__k1;p__p1;c__c1;o__o1;f__f1;g__g1;s__"]
        expected_taxa=["k__k1;p__p1","k__k1;p__p1;c__c1;o__o1;f__f1;g__g1;s__s1",
              "k__k1;p__p1;c__c1;o__o1;f__f1;g__g1"]
        
        self.assertEqual(list(utilities.taxa_remove_unclassified(taxa)), expected_taxa)
        
    def test_taxa_remove_unclassified_spaces(self):
        """ Test the taxa remove unclassified function with spaces in names """
        
        taxa=["k__k1;p__p1 ; c__ ",
            "k__k1;p__p1;c__c1 ; o__o1 ;f__f1;g__g1 ; s__s1 ",
            "k__k1;p__p1;c__c1;o__o1;f__f1;g__g1;s__"]
        expected_taxa=["k__k1;p__p1",
            "k__k1;p__p1;c__c1;o__o1;f__f1;g__g1;s__s1",
            "k__k1;p__p1;c__c1;o__o1;f__f1;g__g1"]
        
        self.assertEqual(list(utilities.taxa_remove_unclassified(taxa)), expected_taxa)

    def test_taxa_by_level(self):
        """ Test the taxa by level function """
        
        taxa=["k__k1;p__p1;c__c1","k__k1;p__p1;c__c2","k__k1;p__p2;c__c3"]
        data=[[1,2],[1,2],[1,2]]
        expected_taxa=["k__k1;p__p1","k__k1;p__p2"]
        expected_data=[[2,4],[1,2]]
        
        actual_taxa, actual_data = utilities.taxa_by_level(taxa, data, 1)
        
        self.assertEqual(actual_taxa, expected_taxa)
        self.assertEqual(actual_data, expected_data)
        
    def test_taxa_by_level_kingdom(self):
        """ Test the taxa by level function set to merge to kingdom level"""
        
        taxa=["k__k1;p__p1;c__c1","k__k1;p__p1;c__c2","k__k1;p__p2;c__c3"]
        data=[[1,2],[1,2],[1,2]]
        expected_taxa=["k__k1"]
        expected_data=[[3,6]]
        
        actual_taxa, actual_data = utilities.taxa_by_level(taxa, data, 0)
        
        self.assertEqual(actual_taxa, expected_taxa)
        self.assertEqual(actual_data, expected_data)
        
    def test_taxa_by_level_strain(self):
        """ Test the taxa by level function set to merge to strain level"""
        
        taxa=["k__k1;p__p1;c__c1;o__o1;f__f1;g__g1;s__s1;t__t1","k__k1;p__p1;c__c2"]
        data=[[1,2],[1,2]]
        expected_taxa=["k__k1;p__p1;c__c1;o__o1;f__f1;g__g1;s__s1;t__t1"]
        expected_data=[[1,2]]
        
        actual_taxa, actual_data = utilities.taxa_by_level(taxa, data, 7)
        
        self.assertEqual(actual_taxa, expected_taxa)
        self.assertEqual(actual_data, expected_data)
        
    def test_taxa_by_level_strain_missing(self):
        """ Test the taxa by level function set to merge to strain level
            though there is no strain level input data """
        
        taxa=["k__k1;p__p1;c__c1;o__o1;f__f1;g__g1;s__s1","k__k1;p__p1;c__c2"]
        data=[[1,2],[1,2]]
        expected_taxa=[]
        expected_data=[]
        
        actual_taxa, actual_data = utilities.taxa_by_level(taxa, data, 7)
        
        self.assertEqual(actual_taxa, expected_taxa)
        self.assertEqual(actual_data, expected_data)

    def test_taxa_by_level_unknown(self):
        """ Test the taxa by level function with one taxon that does not include that level"""
        
        taxa=["k__k1;p__p1","k__k1;p__p1;c__c2","k__k1;p__p2;c__c3"]
        data=[[1,2],[1,2],[1,2]]
        expected_taxa=["k__k1;p__p1;c__c2","k__k1;p__p2;c__c3"]
        expected_data=[[1,2],[1,2]]
        
        actual_taxa, actual_data = utilities.taxa_by_level(taxa, data, 2)
        
        self.assertEqual(sorted(actual_taxa), sorted(expected_taxa))
        self.assertEqual(actual_data, expected_data)
        
    def test_relative_abundance(self):
        """ Test the relative abundance function """
        
        data=[[1,2,3],[1,2,3],[1,1,1]]
        expected_relab=[[1/3.0,2/5.0,3/7.0],[1/3.0,2/5.0,3/7.0],[1/3.0,1/5.0,1/7.0]]
        
        self.assertEqual(utilities.relative_abundance(data),expected_relab)

    def test_relative_abundance_zeros(self):
	    """ Test the relative abundance function with a sample with all zeros"""
	    
	    data=[[1,2,3,0],[1,2,3,0],[1,1,1,0]]
	    expected_relab=[[1/3.0,2/5.0,3/7.0,0],[1/3.0,2/5.0,3/7.0,0],[1/3.0,1/5.0,1/7.0,0]]
	    
	    self.assertEqual(utilities.relative_abundance(data),expected_relab)
        
    def test_taxa_shorten_name(self):
        """ Test the taxa shorten name function """
        
        taxa=["k__k1;p__p1;c__c1","k__k1;p__p1;c__c1;o__o1","k__k1;p__p1"]
        expected_taxa=["p__p1","p__p1","p__p1"]
        
        self.assertEqual(utilities.taxa_shorten_name(taxa, 1), expected_taxa)
        
    def test_taxa_shorten_name_remove_identifier(self):
        """ Test the taxa shorten name function with removing the identifier"""
        
        taxa=["k__k1;p__p1;c__c1","k__k1;p__p1;c__c1;o__o1","k__k1;p__p1"]
        expected_taxa=["p1","p1","p1"]
        
        self.assertEqual(utilities.taxa_shorten_name(taxa, 1, remove_identifier=True), expected_taxa)
        
    def test_terminal_taxa(self):
        """ Test the terminal taxa function """
        
        taxa=["k__k1;p__p1","k__k1;p__p1;c__c1","k__k2;p__p2","k__k3;p__p3;c__c2;o__o3"]
        data=[[1],[2],[3],[4]]
        
        # it is expected that order of the taxa will stay the same as the original input
        expected_taxa=["k__k1;p__p1;c__c1","k__k2;p__p2","k__k3;p__p3;c__c2;o__o3"]
        expected_data=[[2],[3],[4]]
        
        actual_taxa, actual_data = utilities.terminal_taxa(taxa, data)
        
        self.assertEqual(actual_taxa,expected_taxa)
        self.assertEqual(actual_data,expected_data)
        
    def test_terminal_taxa_duplicate(self):
        """ Test the terminal taxa function with two duplicate terminal taxa"""
        
        taxa=["k__k1;p__p1","k__k1;p__p1;c__c1","k__k2;p__p2","k__k3;p__p3;c__c2;o__o3","k__k3;p__p3;c__c2;o__o3"]
        data=[[1],[2],[3],[4],[5]]
        
        # it is expected that order of the taxa will stay the same as the original input
        expected_taxa=["k__k1;p__p1;c__c1","k__k2;p__p2","k__k3;p__p3;c__c2;o__o3"]
        expected_data=[[2],[3],[9]]
        
        actual_taxa, actual_data = utilities.terminal_taxa(taxa, data)
        
        self.assertEqual(actual_taxa,expected_taxa)
        self.assertEqual(actual_data,expected_data)

    def test_terminal_taxa_unclassified_family(self):
        """ Test the terminal taxa function with a taxon with unclassified at family level"""
        
        taxa=["k__k1;p__p1","k__k1;p__p1;c__c1","k__k2;p__p2","k__k3;p__p3;c__c2;o__o3;f__;g__;s__","k__k3;p__p3;c__c2;o__o3;f__f1"]
        data=[[1],[2],[3],[4],[5]]
        
        # it is expected that order of the taxa will stay the same as the original input
        expected_taxa=["k__k1;p__p1;c__c1","k__k2;p__p2","k__k3;p__p3;c__c2;o__o3;f__;g__;s__","k__k3;p__p3;c__c2;o__o3;f__f1"]
        expected_data=[[2],[3],[4],[5]]
        
        actual_taxa, actual_data = utilities.terminal_taxa(taxa, data)
        
        self.assertEqual(actual_taxa,expected_taxa)
        self.assertEqual(actual_data,expected_data)
        
    def test_terminal_taxa_unclassified_species(self):
        """ Test the terminal taxa function with a taxon with unclassified at species level"""
        
        taxa=["k__k3;p__p3;c__c2;o__o3;f__f1;g__g1;s__","k__k3;p__p3;c__c2;o__o3;f__f1;g__g1;s__s1"]
        data=[[1],[2]]
        
        # it is expected that order of the taxa will stay the same as the original input
        expected_taxa=["k__k3;p__p3;c__c2;o__o3;f__f1;g__g1;s__","k__k3;p__p3;c__c2;o__o3;f__f1;g__g1;s__s1"]
        expected_data=[[1],[2]]
        
        actual_taxa, actual_data = utilities.terminal_taxa(taxa, data)
        
        self.assertEqual(actual_taxa,expected_taxa)
        self.assertEqual(actual_data,expected_data)
        
    def test_filter_zero_rows(self):
        """ Test the filter zero rows function """
        
        taxa=["k__k1;p__p1","k__k1;p__p1;c__c1","k__k2;p__p2","k__k3;p__p3;c__c2;o__o3","k__k3;p__p3;c__c2;o__o3"]
        data=[[1],[0],[3],[0],[5]]
        
        # it is expected that order of the taxa will stay the same as the original input
        expected_taxa=["k__k1;p__p1","k__k2;p__p2","k__k3;p__p3;c__c2;o__o3"]
        expected_data=[[1],[3],[5]]
        
        actual_taxa, actual_data = utilities.filter_zero_rows(taxa, data)
        
        self.assertEqual(actual_taxa,expected_taxa)
        self.assertEqual(actual_data,expected_data)     
        
    def test_sort_data(self):
        """ Test the sort data function """
        
        samples=["s1","s3","s2"] 
        data=[1,3,2] 
        
        expected_samples=["s3","s2","s1"]
        expected_data=[3,2,1]
        
        actual_samples, actual_data = utilities.sort_data(data, samples)
        
        self.assertEqual(actual_data, expected_data)
        self.assertEqual(actual_samples, expected_samples)
        
    def test_sort_data_list_of_lists(self):
        """ Test the sort data function with data as a list of lists """
        
        samples=["s1","s3","s2"] 
        data=[[1],[3],[2]] 
        
        expected_samples=["s3","s2","s1"]
        expected_data=[3,2,1]
        
        actual_samples, actual_data = utilities.sort_data(data, samples)

        self.assertEqual(actual_data, expected_data)
        self.assertEqual(actual_samples, expected_samples)
        
    def test_name_files(self):
        """ Test the name files function """
        
        samples=["sample1","sample2"]
        
        self.assertEqual(sorted(utilities.name_files(samples,"/tmp")), ["/tmp/sample1","/tmp/sample2"])
        
    def test_name_files_single_file(self):
        """ Test the name files function with a single file to make sure a string instead of list is returned """
        
        self.assertEqual(utilities.name_files("sample1","/tmp"), "/tmp/sample1")
        
    def test_taxonomy_trim(self):
        """ Test the taxonomy trim function """
        
        taxa = ["k__k3;p__p3;c__c2;o__o3;f__;g__;s__","k__k3;p__p3;c__c2;o__o3;f__f1;g__g1;s__",
                "k__k3;p__p3;c__c2;o__o3;f__f1;g__g1;s__s1","k__k3;p__p3;c__c2;o__o3;f__f1;g__;s__"]
        
        expected_taxa = ["o__o3.f__.g__.s__","g__g1.s__","g__g1.s__s1","f__f1.g__.s__"]
        
        self.assertEqual(utilities.taxonomy_trim(taxa), expected_taxa)
        
    def test_sample_names(self):
        """ Test the sample names function """
        
        files=["s1.fastq","s2.fastq"]
        
        self.assertEqual(utilities.sample_names(files,".fastq"), ["s1","s2"])
        
    def test_sample_names_extension(self):
        """ Test the sample names function without leading period"""
        
        files=["s1.fastq","s2.fastq"]
        
        self.assertEqual(utilities.sample_names(files,"fastq"), ["s1","s2"])
        
    def test_sample_names_period(self):
        """ Test the sample names function with period included in name"""
        
        files=["s1.1.fastq","s2.1.fastq"]
        
        self.assertEqual(utilities.sample_names(files,"fastq"), ["s1.1","s2.1"])
        
    def test_sample_names_pair_identifier(self):
        """ Test the sample names function with a pair identifier """
        
        files=["s1.R1.fastq","s2.R1.fastq"]
        
        self.assertEqual(utilities.sample_names(files,".fastq",".R1"), ["s1","s2"])
        
    def test_sample_names_pair_identifier_not_found(self):
        """ Test the sample names function with a pair identifier that is not included in the names """
        
        files=["s1.R1.fastq","s2.R1.fastq"]
        
        self.assertEqual(utilities.sample_names(files,".fastq","_R1"), ["s1.R1","s2.R1"])
        
    def test_paired_files(self):
        """ Test the paired files function """
        
        files=["s1.R1.fastq","s1.R2.fastq","s2.R1.fastq","s2.R2.fastq"]
        
        expected_pairs = [["s1.R1.fastq","s2.R1.fastq"],["s1.R2.fastq","s2.R2.fastq"]]
        
        actual_pairs = utilities.paired_files(files, ".fastq", pair_identifier=".R1")
        
        self.assertEqual(expected_pairs[0],actual_pairs[0])
        self.assertEqual(expected_pairs[1],actual_pairs[1])

    def test_paired_files_period(self):
        """ Test the paired files function with periods in sample name"""
        
        files=["s1.1.R1.fastq","s1.1.R2.fastq","s2.1.R1.fastq","s2.1.R2.fastq"]
        
        expected_pairs = [["s1.1.R1.fastq","s2.1.R1.fastq"],["s1.1.R2.fastq","s2.1.R2.fastq"]]
        
        actual_pairs = utilities.paired_files(files, ".fastq", pair_identifier=".R1")
        
        self.assertEqual(expected_pairs[0],actual_pairs[0])
        self.assertEqual(expected_pairs[1],actual_pairs[1])
        
    def test_sample_names_pair_identifier_duplicate(self):
        """ Test the sample names function with a pair identifier included in the sample name"""
        
        files=["s_1_1.fastq.gz","s1_1.fastq.gz"]
        
        self.assertEqual(utilities.sample_names(files,".fastq.gz","_1"), ["s_1","s1"])
        
    def test_paired_files_duplicate_identifier_1(self):
        """ Test the paired files function with a first identifier duplicated"""
        
        files=["s_1_1.fastq.gz","s_1_2.fastq.gz","s_1_3_1.fastq.gz","s_1_3_2.fastq.gz"]
        
        expected_pairs = [["s_1_1.fastq.gz","s_1_3_1.fastq.gz"],["s_1_2.fastq.gz","s_1_3_2.fastq.gz"]]
        
        actual_pairs = utilities.paired_files(files, ".fastq.gz", pair_identifier="_1")
        
        self.assertEqual(expected_pairs[0],actual_pairs[0])
        self.assertEqual(expected_pairs[1],actual_pairs[1])
        
    def test_paired_files_duplicate_identifier_2(self):
        """ Test the paired files function with a second identifier duplicated
            Also test extension without leading period. """
        
        files=["s_2_1.fastq","s_2_2.fastq","s_2_3_1.fastq","s_2_3_2.fastq"]
        
        expected_pairs = [["s_2_1.fastq","s_2_3_1.fastq"],["s_2_2.fastq","s_2_3_2.fastq"]]
        
        actual_pairs = utilities.paired_files(files, "fastq", pair_identifier="_1")
        
        self.assertEqual(expected_pairs[0],actual_pairs[0])
        self.assertEqual(expected_pairs[1],actual_pairs[1])
        
    def test_paired_files_duplicate_identifier_3(self):
        """ Test the paired files function with pair identified duplicated """
        
        files=["MR100.R1.fastq","MR100.R2.fastq","MR200.R1.fastq","MR200.R2.fastq"]
        
        expected_pairs = [["MR100.R1.fastq","MR200.R1.fastq"],["MR100.R2.fastq","MR200.R2.fastq"]]
        
        actual_pairs = utilities.paired_files(files, "fastq", pair_identifier="R1")
        
        self.assertEqual(expected_pairs[0],actual_pairs[0])
        self.assertEqual(expected_pairs[1],actual_pairs[1])
        
    def test_paired_files_identifier_not_found(self):
        """ Test the paired files function with an identifier that is not found"""
        
        files=["sample-1.R1.fastq","sample-1.R2.fastq","sample-2.R1.fastq","sample-2.R2.fastq"]
        
        expected_pairs = [[],[]]
        
        actual_pairs = utilities.paired_files(files, ".fastq", pair_identifier="_R1.")
        
        self.assertEqual(expected_pairs[0],actual_pairs[0])
        self.assertEqual(expected_pairs[1],actual_pairs[1])
        
    def test_paired_files_identifier_includes_extension(self):
        """ Test the paired files function with an identifier that is not found because
            it includes the period from the file extension"""
        
        files=["sample-1.R1.fastq","sample-1.R2.fastq","sample-2.R1.fastq","sample-2.R2.fastq"]
        
        expected_pairs = [[],[]]
        
        actual_pairs = utilities.paired_files(files, ".fastq", pair_identifier="R1.")
        
        self.assertEqual(expected_pairs[0],actual_pairs[0])
        self.assertEqual(expected_pairs[1],actual_pairs[1])
        
    def test_read_metadata_columns(self):
        """ Test the read metadata function. Metadata file has samples as columns. """
        
        # create a temp taxonomy file (for sample names)
        taxon_file = write_temp("# taxon\tsample1\tsample2\nbug1\t1\t2\n")
        # create a temp metadata file
        metadata_file = write_temp("# feature\tsample1\tsample2\nfeature1\tA\tB\n")
        
        # get the metadata values
        values = utilities.read_metadata(metadata_file,taxon_file)
        
        # remove the temp files
        os.remove(taxon_file)
        os.remove(metadata_file)
        
        expected_values=[["# feature","sample1","sample2"],["feature1","A","B"]]
        self.assertEqual(values,expected_values)
        
    def test_read_metadata_rows(self):
        """ Test the read metadata function. Metadata file has samples as rows. """
        
        # create a temp taxonomy file (for sample names)
        taxon_file = write_temp("# taxon\tsample1\tsample2\nbug1\t1\t2\n")
        # create a temp metadata file
        metadata_file = write_temp("# sample\tfeature1\nsample1\tA\nsample2\tB\n")
        
        # get the metadata values
        values = utilities.read_metadata(metadata_file,taxon_file)
        
        # remove the temp files
        os.remove(taxon_file)
        os.remove(metadata_file)
        
        expected_values=[["# sample","sample1","sample2"],["feature1","A","B"]]
        self.assertEqual(values,expected_values)   
        
    def test_read_metadata_remove_feature(self):
        """ Test the read metadata function. Metadata file has samples as columns.
            Remove a single feature."""
        
        # create a temp taxonomy file (for sample names)
        taxon_file = write_temp("# taxon\tsample1\tsample2\nbug1\t1\t2\n")
        # create a temp metadata file
        metadata_file = write_temp("# feature\tsample1\tsample2\nfeature1\tA\tB\nfeature2\tC\tD\n")
        
        # get the metadata values
        values = utilities.read_metadata(metadata_file,taxon_file,ignore_features=["feature1"])
        
        # remove the temp files
        os.remove(taxon_file)
        os.remove(metadata_file)
        
        expected_values=[["# feature","sample1","sample2"],["feature2","C","D"]]
        self.assertEqual(values,expected_values)  
        
    def test_label_metadata(self):
        """ Test the label_metadata function. Test default labels. """
        
        data=[["# samples","sample1","sample2"],["feature1","1","2.2"],["feature2","A","B"]]
        labels, new_data=utilities.label_metadata(data)   
        
        expected_data=[["# samples","sample1","sample2"],["feature1",1.0,2.2],["feature2","A","B"]]
        expected_labels={"feature1":"con","feature2":"cat"}
        
        self.assertEqual(new_data,expected_data)
        self.assertEqual(labels, expected_labels)
        
    def test_label_metadata_categorical(self):
        """ Test the label_metadata function. Set categorical label. """
        
        data=[["# samples","sample1","sample2"],["feature1","1","2.2"],["feature2","A","B"]]
        labels, new_data=utilities.label_metadata(data, categorical=["feature1"])   
        
        expected_data=[["# samples","sample1","sample2"],["feature1","1","2.2"],["feature2","A","B"]]
        expected_labels={"feature1":"cat","feature2":"cat"}
        
        self.assertEqual(new_data,expected_data)
        self.assertEqual(labels, expected_labels)
        
    def test_label_metadata_continuous(self):
        """ Test the label_metadata function. Set continuous label. """
        
        data=[["# samples","sample1","sample2"],["feature1","1","2.2"],["feature2","A","B"]]
        labels, new_data=utilities.label_metadata(data, continuous=["feature2"])   
        
        expected_data=[["# samples","sample1","sample2"],["feature1",1.0,2.2],["feature2","A","B"]]
        expected_labels={"feature1":"con","feature2":"con"}
        
        self.assertEqual(new_data,expected_data)
        self.assertEqual(labels, expected_labels)
        
    def test_merge_metadata(self):
        """ Test the merge metadata function."""
        
        metadata=[["# samples","s1","s2"],["feature1","A","B"],["feature2",1,2]]
        samples=["s1","s2"]
        values=[["bug1",1,2],["bug2",2,4]]
        
        merged, new_samples=utilities.merge_metadata(metadata, samples, values)
        
        expected=[["feature1","A","B"],["feature2",1,2],["bug1",1,2],["bug2",2,4]]
        expected_samples=["s1","s2"]
        
        self.assertEqual(merged, expected) 
        self.assertEqual(new_samples, expected_samples)    
        
    def test_merge_metadata_values_without_names(self):
        """ Test the merge metadata function. Test values without names option."""
        
        metadata=[["# samples","s1","s2"],["feature1","A","B"],["feature2",1,2]]
        samples=["s1","s2"]
        values=[[1,2],[2,4]]
        
        merged, new_samples=utilities.merge_metadata(metadata, samples, values, values_without_names=True)
        
        expected=[["feature1","A","B"],["feature2",1,2],[1,2],[2,4]]
        expected_samples=["s1","s2"]
        
        self.assertEqual(merged, expected) 
        self.assertEqual(new_samples, expected_samples)  
        
    def test_merge_metadata_reorder(self):
        """ Test the merge metadata function. Test with reordering."""
        
        metadata=[["# samples","s1","s2"],["feature1","A","B"],["feature2",1,2]]
        samples=["s2","s1"]
        values=[["bug1",1,2],["bug2",2,4]]
        
        merged, new_samples=utilities.merge_metadata(metadata, samples, values)
        
        expected=[["feature1","A","B"],["feature2",1,2],["bug1",2,1],["bug2",4,2]]
        expected_samples=["s1","s2"]
        
        self.assertEqual(merged, expected) 
        self.assertEqual(new_samples, expected_samples) 
        
    def test_merge_metadata_less_metadata(self):
        """ Test the merge metadata function. Test with only a single sample metadata."""
        
        metadata=[["# samples","s1"],["feature1","A"],["feature2",1]]
        samples=["s1","s2"]
        values=[["bug1",1,2],["bug2",2,4]]
        
        # suppress warning message
        # Redirect stdout
        original_stdout=sys.stdout
        sys.stdout=open(os.devnull,"w")
        
        merged, new_samples=utilities.merge_metadata(metadata, samples, values)
        
        # Redirect stdout
        sys.stdout=original_stdout
        
        expected=[["feature1","A"],["feature2",1],["bug1",1],["bug2",2]]
        expected_samples=["s1"]
        
        self.assertEqual(merged, expected)  
        self.assertEqual(new_samples, expected_samples)   
        
    def test_merge_metadata_nomatch(self):
        """ Test the merge metadata function. Test with metadata not matching."""
        
        metadata=[["# samples","s3"],["feature1","A"],["feature2",1]]
        samples=["s1","s2"]
        values=[["bug1",1,2],["bug2",2,4]]
        
        # suppress warning message
        # Redirect stdout
        original_stdout=sys.stdout
        sys.stdout=open(os.devnull,"w")
        
        merged, new_samples=utilities.merge_metadata(metadata, samples, values)
        
        # Redirect stdout
        sys.stdout=original_stdout
        
        expected=[["bug1",1,2],["bug2",2,4]]
        
        self.assertEqual(merged, expected) 
        self.assertEqual(new_samples, samples)  
        
    def test_group_samples_by_metadata(self):
        """ Test the group samples by metadata function """
        
        metadata=["feature1","A","B","A","A","B"]
        data=[[1,2,3,4,5],[11,22,33,44,55],[111,222,333,444,555]]
        samples=["s1","s2","s3","s4","s5"]
        
        sorted_data_grouped, sorted_samples_grouped = utilities.group_samples_by_metadata(metadata, data, samples)
        
        expected_samples={"A":["s1","s3","s4"], "B":["s2","s5"]}
        expected_data={"A":[[1,3,4],[11,33,44],[111,333,444]], "B":[[2,5],[22,55],[222,555]]}
        
        self.assertEqual(sorted_data_grouped, expected_data)
        self.assertEqual(sorted_samples_grouped, expected_samples)
        
    def test_read_picard(self):
        """ Test the read picard function. Test with all values passing threshold """
        
        file_text="\n".join(["## htsjdk.samtools.metrics.StringHeader",
            "# picard.analysis.CollectMultipleMetrics INPUT=AYBNT.1.unmapped.bam ASSUME_SORTED=true",
            "# Started on: Thu Jan 01 01:12:33 EST 2017",
            "",
            "",
            "## HISTOGRAM    java.lang.Integer",
            "CYCLE\tMEAN_QUALITY",
            "1\t39.111874",
            "2\t35.614163",
            "3\t35.887343",
            "4\t42.882504",
            "5\t52.643778"])
        
        expected_data=[(1,39.111874),(2,35.614163),(3,35.887343),(4,42.882504),(5,52.643778)]
        
        tmp_file=write_temp(file_text)
        
        data, below_threshold = utilities.read_picard(tmp_file)
        os.remove(tmp_file)
        
        self.assertFalse(below_threshold)
        self.assertEqual(data, expected_data)
        
    def test_read_picard_threshold(self):
        """ Test the read picard function. Test with some values passing threshold """
        
        file_text="\n".join(["## htsjdk.samtools.metrics.StringHeader",
            "# picard.analysis.CollectMultipleMetrics INPUT=AYBNT.1.unmapped.bam ASSUME_SORTED=true",
            "# Started on: Thu Jan 01 01:12:33 EST 2017",
            "",
            "",
            "## HISTOGRAM    java.lang.Integer",
            "CYCLE\tMEAN_QUALITY",
            "1\t39.111874",
            "2\t35.614163",
            "3\t35.887343",
            "4\t42.882504",
            "5\t52.643778"])
        
        expected_data=[(1,39.111874),(2,35.614163),(3,35.887343),(4,42.882504),(5,52.643778)]
        
        tmp_file=write_temp(file_text)
        
        data, below_threshold = utilities.read_picard(tmp_file, threshold=37)
        os.remove(tmp_file)
        
        self.assertTrue(below_threshold)
        self.assertEqual(data, expected_data)
        
    def test_rank_species_average_abundance(self):
        """ Test the rank species average abundance function with a small merged taxonomy file """
        
        file_contents = "\n".join(["#SampleID\tS1\tS2\tS3",
        "k__Archaea\t0.5\t0.5\t0.5", "k__Archaea|p__Euryarchaeota\t0.5\t0.5\t0.5",
        "k__Viruses|p__Viruses_noname|c__Viruses_noname|o__Viruses_noname|f__Retroviridae|g__Gammaretrovirus|s__Murine_osteosarcoma_virus\t0.1\t0.2\t0.3",
        "k__Viruses|p__Viruses_noname|c__Viruses_noname|o__Viruses_noname|f__Retroviridae|g__Gammaretrovirus|s__Murine_osteosarcoma_virus|t__PRJNA14655\t0.2\t0.2\t0.2",
        "g__Alistipes|s__Alistipes_indistinctus\t0.3\t0.4\t0.5",
        "g__Alistipes|s__Alistipes_onderdonkii\t0.4\t0.5\t0.6",
        "g__Alistipes|s__Alistipes_putredinis\t0.1\t0.2\t0.1"])

        expected_species = ["s__Alistipes_onderdonkii","s__Alistipes_indistinctus","s__Murine_osteosarcoma_virus","s__Alistipes_putredinis"]

        temp_file = write_temp(file_contents)
        
        ranked_species = utilities.rank_species_average_abundance(temp_file)
        os.remove(temp_file)
        
        self.assertEqual(ranked_species,expected_species)
        
    def test_order_clade_list(self):
        """ Test the order clade list function """
        
        taxonomy_file_contents = "\n".join(["#SampleID\tS1\tS2\tS3",
        "k__Archaea\t0.5\t0.5\t0.5", "k__Archaea|p__Euryarchaeota\t0.5\t0.5\t0.5",
        "k__Viruses|p__Viruses_noname|c__Viruses_noname|o__Viruses_noname|f__Retroviridae|g__Gammaretrovirus|s__Murine_osteosarcoma_virus\t0.1\t0.2\t0.3",
        "k__Viruses|p__Viruses_noname|c__Viruses_noname|o__Viruses_noname|f__Retroviridae|g__Gammaretrovirus|s__Murine_osteosarcoma_virus|t__PRJNA14655\t0.2\t0.2\t0.2",
        "g__Alistipes|s__Alistipes_indistinctus\t0.3\t0.7\t0.5",
        "g__Alistipes|s__Alistipes_onderdonkii\t0.4\t0.5\t0.6",
        "g__Alistipes|s__Alistipes_putredinis\t0.1\t0.1\t0.1"])

        clades_list_file_contents = "\n".join(["s__Alistipes_putredinis", 
            "s__Alistipes_indistinctus", "s__Murine_osteosarcoma_virus", "s__Acidaminococcus_sp_D21"])

        expected_species = ["s__Alistipes_indistinctus","s__Murine_osteosarcoma_virus","s__Alistipes_putredinis"]

        taxonomy_temp_file = write_temp(taxonomy_file_contents)
        clade_temp_file = write_temp(clades_list_file_contents)
        output_file = write_temp("")
        
        ranked_species = utilities.order_clade_list(None,clade_temp_file,taxonomy_temp_file,output_file)
        os.remove(taxonomy_temp_file)
        os.remove(clade_temp_file)
        
        with open(output_file) as file_handle:
            ranked_species=[line.rstrip() for line in file_handle.readlines()]
        os.remove(output_file)
        
        self.assertEqual(ranked_species,expected_species)        
        
        




#!/usr/bin/env python

"""
biobakery_workflows: databases module
Download databases for workflows

Copyright (c) 2017 Harvard School of Public Health

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import sys
    
from . import utilities
from . import config
from . import data

import argparse
import os
import subprocess
import shutil
import zipfile

def check_dependencies(depends):
    """ Check the required software is installed """
    
    for tool, package in depends:
        try:
            subprocess.check_output(["which",tool],stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError:
            sys.exit("ERROR: Unable to find tool required for database install: "+tool+
                ". Please install "+package+".")

def default_install_location():
    """ Select the default install location based on the permissions """
    
    for folder in config.install_locations():
        # check the base folder is writeable
        if os.access(os.path.dirname(folder), os.W_OK):
            return folder
        
    return ""

def try_create_folder(folder):
    """ Try to create a folder, exit on error """

    try:
        if not os.path.exists(folder):
            os.makedirs(folder)
    except EnvironmentError:
        sys.exit("Unable to create folder: "+ folder)

def run_command(command,shell=False):
    """ Run the command, exit if error """
    
    try:
        id=subprocess.check_call(command, shell=shell)
    except subprocess.CalledProcessError:
        sys.exit("Unable to install database. Error running command: "+" ".join(command))
        
def create_strainphlan_db(location):
    """ Create the strainphlan fasta file from the bowtie2 indexes """
    
    # get the default fasta install folder
    install_folder=os.path.join(location,config.ShotGun.vars["strainphlan_db_markers"].default_folder)
    install_folder_reference=os.path.join(location,config.ShotGun.vars["strainphlan_db_reference"].default_folder)
    
    # create the install folder if it does not exist
    try_create_folder(install_folder)
    try_create_folder(install_folder_reference)
    
    # find the strainphlan db folder and index files
    try:
        strainphlan_db=os.path.join(os.path.dirname(subprocess.check_output(["which","strainphlan.py"])),"metaphlan_databases","mpa_v20_m200")
    except subprocess.CalledProcessError:
        sys.exit("Unable to find strainphlan install.")
        
    # generate the fasta marker files
    print("Generating strainphlan fasta database")
    run_command(" ".join(["bowtie2-inspect",strainphlan_db,">",os.path.join(install_folder,"all_markers.fasta")]),shell=True)

def parse_arguments(args):
    """ 
    Parse the arguments from the user
    """
    parser = argparse.ArgumentParser(
        description= "bioBakery workflows databases\n",
        formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument(
        "--available", 
        action="store_true",
        help="print the available databases\n")
    parser.add_argument(
        "--install",
        metavar="<workflow>",
        choices=["wmgx","wmgx_demo","wmgx_wmtx","16s_usearch","16s_dada2","16s_its","isolate_assembly"],
        help="install the databases for the selected workflow\n")
    parser.add_argument(
        "--location", 
        default=default_install_location(),
        help="location to install databases [DEFAULT: "+default_install_location()+")]\n")
    
    return parser.parse_args()

def main():
    # Parse arguments from the command line
    args=parse_arguments(sys.argv)
    
    # if no option is set, show available databases
    if not args.install and not args.available:
        args.available = True
        
    if args.available:
        print("There are five available database sets each corresponding to a data "+
            "processing workflow.\n"+
            "wmgx: The full databases for the whole metagenome workflow\n"+
            "wmgx_demo: The demo databases for the whole metagenome workflow\n"+
            "wmgx_wmtx: The full databases for the whole metagenome and metatranscriptome workflow\n"+
            "16s_usearch: The full databases for the 16s workflow\n" +
            "16s_dada2: The full databases for the dada2 workflow\n"+
            "16s_its: The unite database for the its workflow\n"+
            "isolate_assembly: The eggnog-mapper databases for the assembly workflow")
        sys.exit(0)
    
    # Check the install location
    if not args.location:
        sys.exit("Please select the install location as the default options are not writable")
        
    if not args.install:
        sys.exit("Please select a database set to install")
        
    # try to create the base install folder
    try_create_folder(args.location)
    
    # check required dependencies are installed for wmgx installs
    dependencies=[]
    if "wmgx" in args.install:
        dependencies=[("humann2_databases","HUMAnN2"),("kneaddata_database","Kneaddata"),
            ("strainphlan.py","StrainPhlAn"),("bowtie2-inspect","Bowtie2")]
    elif "usearch" in args.install:
        dependencies=[("usearch","usearch")]    
    if dependencies:
        check_dependencies(dependencies)
        
    # install humann2 utility dbs for all shotgun workflows
    humann2_install_folder=os.path.join(args.location,"humann2")
    if "wmgx" in args.install:
        print("Installing humann2 utility mapping database")
        run_command(["humann2_databases","--download","utility_mapping","full",humann2_install_folder])
        
        # create the strainphlan fasta database of markers
        create_strainphlan_db(args.location)
        
    # install the databases based on the workflow selected
    if args.install in ["wmgx","wmgx_wmtx"]:
        # install the full chocophlan and uniref90
        print("Installing humann2 nucleotide and protein databases")
        run_command(["humann2_databases","--download","chocophlan","full",humann2_install_folder])
        run_command(["humann2_databases","--download","uniref","uniref90_diamond",humann2_install_folder])
        
        # install the two kneaddata databases
        print("Installing hg kneaddata database")
        run_command(["kneaddata_database","--download","human_genome","bowtie2",
            os.path.join(args.location,config.ShotGun.vars["kneaddata_db_human_genome"].default_folder)])
        
    elif args.install == "wmgx_demo":
        # install the demo chocophlan and demo uniref90
        print("Installing humann2 DEMO nucleotide and protein databases")
        run_command(["humann2_databases","--download","chocophlan","DEMO",humann2_install_folder])
        run_command(["humann2_databases","--download","uniref","DEMO_diamond",humann2_install_folder])
        
        # Install demo kneaddata databases from examples folder to install folder
        print("Installing DEMO hg kneaddata database")
        shutil.copytree(data.get_kneaddata_hg_demo_folder(),
            os.path.join(args.location,config.ShotGun.vars["kneaddata_db_human_genome"].default_folder))
        
    elif args.install == "16s_usearch":
        # download the green genes fasta and taxonomy files
        print("Downloading green genes database files")
        usearch_fasta_install_path=os.path.join(args.location,config.SixteenS.vars["greengenes_fasta"].default_path)
        usearch_database_install_path=os.path.join(args.location,config.SixteenS.vars["greengenes_usearch"].default_path)
        utilities.download_file(config.SixteenS.vars["greengenes_fasta"].url,
            usearch_fasta_install_path)
        # use the fasta file also as the database so install works for both usearch and vsearch
        try_create_folder(os.path.dirname(usearch_database_install_path))
        shutil.copy(usearch_fasta_install_path,usearch_database_install_path)
        utilities.download_file(config.SixteenS.vars["greengenes_taxonomy"].url,
            os.path.join(args.location,config.SixteenS.vars["greengenes_taxonomy"].default_path))
        
    elif args.install == "16s_dada2":
        # download the green genes fasta and taxonomy files
        print("Downloading dada2 green genes database files")
        dada2_install_path=os.path.join(args.location,config.SixteenS.vars["greengenes_dada2"].default_path)
        utilities.download_file(config.SixteenS.vars["greengenes_dada2"].url,
            dada2_install_path)
        utilities.download_file(config.SixteenS.vars["rdp_dada2"].url,
            os.path.join(args.location,config.SixteenS.vars["rdp_dada2"].default_path))
        utilities.download_file(config.SixteenS.vars["silva_dada2"].url,
            os.path.join(args.location,config.SixteenS.vars["silva_dada2"].default_path))
        utilities.download_file(config.SixteenS.vars["rdp_species_dada2"].url,
            os.path.join(args.location,config.SixteenS.vars["rdp_species_dada2"].default_path))
        utilities.download_file(config.SixteenS.vars["silva_species_dada2"].url,
            os.path.join(args.location,config.SixteenS.vars["silva_species_dada2"].default_path))

    elif args.install == "16s_its":
        # download unite database for its workflow
        print("Downloading UNITE database files")
        its_install_path = os.path.join(args.location, config.SixteenS.vars["unite_zip"].default_path)
        utilities.download_file(config.SixteenS.vars["unite_zip"].url,
                                its_install_path)
        zip_ref = zipfile.ZipFile(os.path.join(args.location,config.SixteenS.vars["unite_zip"].default_path), 'r')
        zip_ref.extractall(os.path.join(args.location,config.SixteenS.vars["unite"].default_folder))
        zip_ref.close()

    elif args.install == "isolate_assembly":
        print("Downloading eggnog mapper databases")
        eggnog_install_path = os.path.join(args.location, "eggnog_mapper/")
        try_create_folder(eggnog_install_path)
        run_command(["download_eggnog_data.py","--data_dir",eggnog_install_path,"-y"])
        run_command(["ln","-s",eggnog_install_path+"/eggnog.db","/opt/conda/bin/data/"])
        run_command(["ln","-s",eggnog_install_path+"/eggnog_proteins.dmnd","/opt/conda/bin/data/"])

    if "16s" in args.install:
        # install the picrust databases
        run_command(["download_picrust_files.py"])
       
    # if metatranscriptome workflow, install the additional kneaddata database
    if args.install == "wmgx_wmtx":
        print("Installing rRNA and mRNA kneaddata database")
        run_command(["kneaddata_database","--download","ribosomal_RNA","bowtie2",
            os.path.join(args.location,config.ShotGun.vars["kneaddata_db_rrna"].default_folder)])
        run_command(["kneaddata_database","--download","human_transcriptome","bowtie2",
            os.path.join(args.location,config.ShotGun.vars["kneaddata_db_human_metatranscriptome"].default_folder)])
        
    # Check for a custom install location
    if args.location != default_install_location():
        print("\n\nA custom install location was selected. Please set the "+
            "environment variable $"+config.Workflow.base_environment_variable+" to the install location.")
    
